#!/usr/bin/env python

import timeit
import sys

if __name__ == '__main__':
    runStatement = """
for i in xrange(0, 100000):
    entry = db.loadRecordByName(random.choice(keys))
"""

    setupStatement = """
import os, sys
import random
import mydb
db = mydb.mydb()
keys = db.keys()
"""

    t = timeit.Timer(runStatement, setupStatement)

    print "[MYSQL TIMEIT]"
    print t.repeat(2, 1)



#!/usr/bin/env python
# Copyright (c) 2016, The Regents of the University of California.

import sys
import timeit

if __name__ == '__main__':
    if len(sys.argv) != 3:
        print "Usage: %s <filename> <fa/fq>" % sys.argv[0]
        exit(1)

    filename = sys.argv[1]
    fafq = sys.argv[2]

    fqrunStatement = """
createscreed.create_db(filename, fastq.FieldTypes, iterfunc)
theFile.close()
"""

    fqsetupStatement = """
import os, sys
thisdir = sys.path[0]
libdir = os.path.abspath(os.path.join(thisdir, '..', 'screed'))
sys.path.insert(0, libdir)
import createscreed
import fastq
FASTQFIELDTYPES = ('name', 'annotations', 'sequence', 'quality')
filename = '%s'
theFile = open(filename, 'rb')
iterfunc = fastq.fastq_iter(theFile)
""" % filename

    farunStatement = """
createscreed.create_db(filename, fasta.FieldTypes, iterfunc)
theFile.close()
"""

    fasetupStatement = """
import os, sys
thisdir = sys.path[0]
libdir = os.path.abspath(os.path.join(thisdir, '..', 'screed'))
sys.path.insert(0, libdir)
import createscreed
import fasta
FASTAFIELDTYPES = ('name', 'description', 'sequence')
filename = '%s'
theFile = open(filename, 'rb')
iterfunc = fasta.fasta_iter(theFile)
""" % filename

    t = None
    if fafq == 'fasta':
        t = timeit.Timer(farunStatement, fasetupStatement)
    elif fafq == 'fastq':
        t = timeit.Timer(fqrunStatement, fqsetupStatement)
    else:
        raise ValueError("Invalid db type specified: %s" % fafq)

    print "[SCREED CREATE]%s:" % filename
    print t.repeat(2, 1)



#
# This file is part of screed, http://github.com/dib-lab/screed/, and is
# Copyright (C) Michigan State University, 2009-2015. It is licensed under
# the three-clause BSD license; see doc/LICENSE.txt.
# Contact: khmer-project@idyll.org
#
# This file has been modified from the khmer project at
# https://github.com/dib-lab/khmer/blob/a8356b7abbebf8540c7656378b1459442b781f87/tests/khmer_tst_utils.py
#

import tempfile
import os
import shutil
from io import StringIO
import sys
import traceback

from importlib import resources

# Remove when we drop support for 3.8
if sys.version_info < (3, 9):
    import importlib_resources as resources


def get_test_data(filename):
    filepath = resources.files('screed') / 'screed' / 'tests' / filename
    if not filepath.exists() or not os.path.isfile(filepath):
        filepath = os.path.join(os.path.dirname(__file__), 'test-data',
                                filename)
    return filepath


cleanup_list = []


def get_temp_filename(filename, tempdir=None):
    if tempdir is None:
        tempdir = tempfile.mkdtemp(prefix='screedtest_')
        cleanup_list.append(tempdir)

    return os.path.join(tempdir, filename)


def cleanup():
    global cleanup_list

    for path in cleanup_list:
        shutil.rmtree(path, ignore_errors=True)
    cleanup_list = []



import mdbConstants
import MySQLdb
import UserDict
import types

class _mdb_record_dict(UserDict.DictMixin):
    """
    Simple dict-like record interface with bag behavior.
    """
    def __init__(self, *args, **kwargs):
        self.d = dict(*args, **kwargs)
        
    def __getitem__(self, name):
        return self.d[name]

    def __setitem__(self, name, value):
        self.d[name] = value
    
    def __getattr__(self, name):
        try:
            return self.d[name]
        except KeyError:
            raise AttributeError, name

    def keys(self):
        return self.d.keys()

class mydb(object):
    def __init__(self):
        self._conn = MySQLdb.connect(db=mdbConstants._DBNAME, user=mdbConstants._USER)
                                       
        cur = self._conn.cursor()
        cur.execute('SELECT id, fieldname FROM %s' % mdbConstants._SCREEDADMIN)
        self._adm = dict(cur.fetchall())
        keys = self._adm.keys()
        keys.sort()

        self._fields = self._adm.values()
        self._fields.insert(0, mdbConstants._PRIMARY_KEY.lower())
        self._fieldStr = ",".join(self._fields)

        self._queryBy = self._adm[keys[0]]

    def close(self):
        """
        Closes the database handles
        """
        self._conn.close()

    def loadRecordByIndex(self, idx):
        """
        Loads a record from the database by index
        """
    
    def loadRecordByName(self, key):
        """
        As above, by name
        """
        cursor = self._conn.cursor()
        query = "SELECT %s FROM %s WHERE %s='%s'" % (self._queryBy,
                                                     mdbConstants._DICT_TABLE,
                                                     self._queryBy,
                                                     key)
        cursor.execute(query)
        if type(cursor.fetchone()) == types.NoneType:
            raise KeyError("Key %s not found" % key)

        query = "SELECT %s FROM %s WHERE %s='%s'" % (self._fieldStr,
                                                     mdbConstants._DICT_TABLE,
                                                     self._queryBy,
                                                     key)
        cursor.execute(query)
        return _mdb_record_dict(zip(self._fields, cursor.fetchone()))

    def keys(self):
        """
        Returns a list of keys in database
        """
        cursor = self._conn.cursor()
        query = "SELECT %s FROM %s" % (self._queryBy,
                                       mdbConstants._DICT_TABLE)
        cursor.execute(query)
        return [elem for elem, in cursor]



import os
import MySQLdb
import mdbConstants

def create_db(fields, rcrditer):
    """
    Populates the mysql database with records from the record iter
    """
    conn = MySQLdb.connect(db=mdbConstants._DBNAME, user=mdbConstants._USER)

    cur = conn.cursor()

    # Create the admin table
    cur.execute('CREATE TABLE %s (ID int NOT NULL auto_increment, '\
               'FIELDNAME TEXT, PRIMARY KEY(ID))' % mdbConstants._SCREEDADMIN)

    for attribute in fields:
        cur.execute("INSERT INTO %s (FIELDNAME) VALUES ('%s')" % \
            (mdbConstants._SCREEDADMIN, attribute))

    # Setup the dictionary table creation field substring
    otherFields = fields[1:]
    createsub = ['%s TEXT' % field for field in otherFields]
    createsub.insert(0, '%s VARCHAR(100)' % fields[0])
    createsub = ','.join(createsub)

    # Create the dictionary table
    cur.execute('CREATE TABLE %s (%s int NOT NULL auto_increment, %s, PRIMARY KEY(%s))' %
                (mdbConstants._DICT_TABLE, mdbConstants._PRIMARY_KEY,
                 createsub,
                 mdbConstants._PRIMARY_KEY))

    # Attribute to index
    queryby = fields[0]

    # Make the index on the 'queryby' attribute
    cur.execute('CREATE UNIQUE INDEX %sidx ON %s(%s)' %
                (queryby, mdbConstants._DICT_TABLE, queryby))

    # Setup the 'perc' pgres substring
    perc = ', '.join(['%s' for i in range(len(fields))])

    # Setup the sql substring for inserting data into db
    fieldsub = ','.join(fields)

    # Pull data from rcrditer and store in database
    for record in rcrditer:
        data = tuple([record[key] for key in fields])
        cur.execute('INSERT INTO %s (%s) VALUES (%s)' %\
                    (mdbConstants._DICT_TABLE, fieldsub, perc),
                    data)

    conn.commit()
    cur.close()
    conn.close()

def droptables():
    """
    Drops tables in db 
    """
    conn = MySQLdb.connect(db=mdbConstants._DBNAME, user=mdbConstants._USER)

    cur = conn.cursor()

    try:
        cur.execute('DROP TABLE %s;' % mdbConstants._DICT_TABLE)
    except:
        pass
    try:
        cur.execute('DROP TABLE %s;' % mdbConstants._SCREEDADMIN)
    except:
        pass

    conn.commit()
    cur.close()
    conn.close()



# Copyright (c) 2016, The Regents of the University of California.

from __future__ import absolute_import
from . import DBConstants
from .screedRecord import Record
from .utils import to_str

FieldTypes = (('name', DBConstants._INDEXED_TEXT_KEY),
              ('annotations', DBConstants._STANDARD_TEXT),
              ('sequence', DBConstants._STANDARD_TEXT),
              ('quality', DBConstants._STANDARD_TEXT))


def fastq_iter(handle, line=None, parse_description=False):
    """
    Iterator over the given FASTQ file handle returning records. handle
    is a handle to a file opened for reading
    """
    if line is None:
        line = handle.readline()
    line = to_str(line.strip())
    while line:
        data = {}

        if line and not line.startswith('@'):
            raise IOError("Bad FASTQ format: no '@' at beginning of line")

        # Try to grab the name and (optional) annotations
        if parse_description:
            try:
                data['name'], data['annotations'] = line[1:].split(' ', 1)
            except ValueError:  # No optional annotations
                data['name'] = line[1:]
                data['annotations'] = ''
                pass
        else:
            data['name'] = line[1:]
            data['annotations'] = ''

        # Extract the sequence lines
        sequence = []
        line = to_str(handle.readline().strip())
        while line and not line.startswith('+') and not line.startswith('#'):
            sequence.append(line)
            line = to_str(handle.readline().strip())

        data['sequence'] = ''.join(sequence)

        # Extract the quality lines
        quality = []
        line = to_str(handle.readline().strip())
        seqlen = len(data['sequence'])
        aclen = 0
        while not line == '' and aclen < seqlen:
            quality.append(line)
            aclen += len(line)
            line = to_str(handle.readline().strip())

        data['quality'] = ''.join(quality)
        if len(data['sequence']) != len(data['quality']):
            raise IOError('sequence and quality strings must be '
                          'of equal length')

        yield Record(**data)



#!/usr/bin/env python
# Copyright (c) 2016, The Regents of the University of California.
import sys
import os

class fastaModel(object):
    """
    Contains methods for writing data to a file in the fasta format
    """
    def __init__(self, fileHandle):
        self.fileHandle = fileHandle
        self.currSeq = ""

    def writeName(self, name):
        """
        Writes the given name to the fileHandle in the fasta format
        """
        self.fileHandle.write(">%s " % name.strip())

    def writeDescription(self, description):
        """
        Writes the given description and the stored sequence to the file
        """
        self.fileHandle.write("%s\n%s\n" % (description.strip(), self.currSeq))

    def writeSequence(self, sequence):
        """
        Stores the given sequence until a call to writeDescription is made
        so that the description and sequence will be stored in the correct
        fasta order
        """
        self.currSeq = sequence.strip()

def convertFastqToFasta(inputFilename, outputFilename):
    """
    Converts the given fastq file (inputFilename) to an equilivalent fasta file
    (outputFilename). The fastq's quality information is converted to a fasta's
    'description' field. Sequence and name fields are left alone
    """

    inputFile = open(inputFilename, "rb")
    outputFile = open(outputFilename, "wb")

    model = fastaModel(outputFile)

    for line in inputFile:
        if line.startswith("@"): # Line is a name
            model.writeName(line[1:])
        elif line.startswith('+'): # Next line is the quality
            quality = inputFile.next()
            model.writeDescription(quality)
        else: # Line is the sequence
            model.writeSequence(line)

    outputFile.close()

if __name__ == '__main__':
    if len(sys.argv) != 3:
        print "Usage: <input filename> <output filename>"
        exit(1)

    inputFilename = sys.argv[1]
    outputFilename = sys.argv[2]

    if not os.path.isfile(inputFilename):
        print "Error: %s doesn't exist" % inputFilename
        exit(2)

    convertFastqToFasta(inputFilename, outputFilename)



# Copyright (c) 2016, The Regents of the University of California.

from __future__ import absolute_import

import argparse
import itertools
import os
try:
    import sqlite3
except ImportError:
    pass
import itertools
import sys

from . import DBConstants, fasta, fastq, openscreed


def create_db(filepath, fields, rcrditer):
    """
    Creates a screed database in the given filepath. Fields is a tuple
    specifying the names and relative order of attributes in a
    record. rcrditer is an iterator returning records over a
    sequence dataset. Records yielded are in dictionary form
    """
    try:
        sqlite3
    except NameError:
        raise Exception("error: sqlite3 is needed for this functionality" +
                        " but is not installed.")

    if not filepath.endswith(DBConstants.fileExtension):
        filepath += DBConstants.fileExtension

    if os.path.exists(filepath):  # Remove existing files
        os.unlink(filepath)

    con = sqlite3.connect(filepath)
    cur = con.cursor()

    # Sqlite PRAGMA settings for speed
    cur.execute("PRAGMA synchronous='OFF'")
    cur.execute("PRAGMA locking_mode=EXCLUSIVE")

    # Create the admin table
    cur.execute('CREATE TABLE %s (%s INTEGER PRIMARY KEY, '
                '%s TEXT, %s TEXT)' % (DBConstants._SCREEDADMIN,
                                       DBConstants._PRIMARY_KEY,
                                       DBConstants._FIELDNAME,
                                       DBConstants._ROLENAME))
    query = 'INSERT INTO %s (%s, %s) VALUES (?, ?)' % \
            (DBConstants._SCREEDADMIN, DBConstants._FIELDNAME,
             DBConstants._ROLENAME)

    # Put the primary key in as an attribute
    cur.execute(query, (DBConstants._PRIMARY_KEY,
                        DBConstants._PRIMARY_KEY_ROLE))
    for attribute, role in fields:
        cur.execute(query, (attribute, role))

    # Setup the dictionary table creation field substring
    fieldsub = ','.join(['%s TEXT' % field for field, role in fields])

    # Create the dictionary table
    cur.execute('CREATE TABLE %s (%s INTEGER PRIMARY KEY, %s)' %
                (DBConstants._DICT_TABLE, DBConstants._PRIMARY_KEY,
                 fieldsub))

    # Setup the 'qmarks' sqlite substring
    qmarks = ','.join(['?' for i in range(len(fields))])

    # Setup the sql substring for inserting fields into database
    fieldsub = ','.join([fieldname for fieldname, role in fields])

    query = 'INSERT INTO %s (%s) VALUES (%s)' %\
            (DBConstants._DICT_TABLE, fieldsub, qmarks)
    # Pull data from the iterator and store in database
    # Commiting in batches seems faster than a single call to executemany
    data = (tuple(record[fieldname] for fieldname, role in fields)
            for record in rcrditer)
    while True:
        batch = list(itertools.islice(data, 10000))
        if not batch:
            break
        cur.executemany(query, batch)
    con.commit()

    # Attribute to index
    queryby = fields[0][0]  # Defaults to the first field
    for fieldname, role in fields:
        if role == DBConstants._INDEXED_TEXT_KEY:
            queryby = fieldname
            break

    # Make the index on the 'queryby' attribute
    cur.execute('CREATE UNIQUE INDEX %sidx ON %s(%s)' %
                (queryby, DBConstants._DICT_TABLE, queryby))

    con.commit()
    con.close()


def make_db(filename):
    iterfunc = openscreed.Open(filename, parse_description=True)

    field_mapping = {
        fastq.fastq_iter.__name__: fastq.FieldTypes,
        fasta.fasta_iter.__name__: fasta.FieldTypes
    }

    fieldTypes = field_mapping[iterfunc.iter_fn.__name__]

    # Create the screed db
    create_db(filename, fieldTypes, iterfunc)


def main(args):
    parser = argparse.ArgumentParser(description="A shell interface to the "
                                     "screed database writing function")
    parser.add_argument('filename')
    args = parser.parse_args(args)

    make_db(args.filename)

    print("Database saved in {}{}".format(args.filename,
                                          DBConstants.fileExtension))
    exit(0)


if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/env python

# Copyright (c) 2008, Michigan State University.
# Copyright (c) 2016, The Regents of the University of California.

from __future__ import print_function

import argparse
import os
import sys

from screed import ToFasta


# Shell interface to the ToFasta screed conversion function
def main(args):
    parser = argparse.ArgumentParser(
        description="Convert a screed database to a FASTA file")
    parser.add_argument('dbfile')
    parser.add_argument('outputfile', default='/dev/stdout', nargs='?')
    args = parser.parse_args(args)

    if not os.path.isfile(args.dbfile):
        print("No such file: %s" % args.dbfile)
        exit(1)

    n = ToFasta(args.dbfile, args.outputfile)

    sys.stderr.write('Wrote {} records in FASTA format.\n'.format(n))


if __name__ == '__main__':
    main(sys.argv[1])



# Copyright (c) 2008-2015, Michigan State University

from __future__ import print_function
from __future__ import absolute_import

import tempfile
import os
import sys
import io
import threading
import subprocess

import pytest

import screed
from . import screed_tst_utils as utils
from . import test_fasta
from . import test_fastq
from screed.DBConstants import fileExtension


def streamer_reader(ifilename, exception):
    try:
        for read in screed.open(ifilename):
            pass
    except Exception as e:
        exception.append(e)


def streamer(ifilename):

    # Get temp filenames, etc.
    in_dir = tempfile.mkdtemp(prefix="screedtest_")
    fifo = os.path.join(in_dir, 'fifo')
    ifile = io.open(ifilename, 'rb')

    # make a fifo to simulate streaming
    os.mkfifo(fifo)

    exception = []
    # FIFOs MUST BE OPENED FOR READING BEFORE THEY ARE WRITTEN TO
    # If this isn't done, they will BLOCK and things will hang.
    # rvalues will hold the return from the threaded function
    thread = threading.Thread(target=streamer_reader, args=[fifo, exception])
    thread.start()

    fifofile = io.open(fifo, 'wb')
    # read binary to handle compressed files
    chunk = ifile.read(8192)
    while len(chunk) > 0:
        fifofile.write(chunk)
        chunk = ifile.read(8192)

    fifofile.close()

    thread.join()

    if len(exception) > 0:
        raise exception[0]


def test_stream_fa():
    streamer(utils.get_test_data('test.fa'))


def test_stream_fq():
    streamer(utils.get_test_data('test.fastq'))


@pytest.mark.xfail()
def test_stream_fa_gz():
    streamer(utils.get_test_data('test.fa.gz'))


def test_stream_gz_fail():
    try:
        streamer(utils.get_test_data('test.fastq.gz'))
        assert 0, "This should not work yet"
    except ValueError as err:
        print(str(err))


@pytest.mark.xfail()
def test_stream_fq_gz():
    streamer(utils.get_test_data('test.fastq.gz'))


def test_stream_fa_bz2():
    streamer(utils.get_test_data('test.fa.bz2'))


def test_stream_fq_bz2():
    streamer(utils.get_test_data('test.fastq.bz2'))



# Copyright (c) 2008, Michigan State University.
"""Reader and writer for screed."""

from __future__ import absolute_import

import os
import io
import sys
import gzip
import bz2
from collections.abc import MutableMapping

try:
    import sqlite3
except ImportError:
    pass

from . import DBConstants
from . import screedRecord
from .fastq import fastq_iter
from .fasta import fasta_iter
from .utils import to_str


def _normalize_filename(filename):
    """Map '-' to '/dev/stdin' to handle the usual shortcut."""
    if filename == '-':
        filename = '/dev/stdin'
    return filename


class Open(object):
    def __init__(self, filename, *args, **kwargs):
        self.sequencefile = None
        self.iter_fn = self.open_reader(filename, *args, **kwargs)
        if self.iter_fn:
            self.__name__ = self.iter_fn.__name__

    def open_reader(self, filename, *args, **kwargs):
        """
        Make a best-effort guess as to how to parse the given sequence file.

        Handles '-' as shortcut for stdin.
        Deals with .gz, FASTA, and FASTQ records.
        """
        magic_dict = {
            b"\x1f\x8b\x08": "gz",
            b"\x42\x5a\x68": "bz2",
            # "\x50\x4b\x03\x04": "zip"
        }  # Inspired by http://stackoverflow.com/a/13044946/1585509
        filename = _normalize_filename(filename)
        bufferedfile = io.open(file=filename, mode='rb', buffering=8192)
        num_bytes_to_peek = max(len(x) for x in magic_dict)
        file_start = bufferedfile.peek(num_bytes_to_peek)
        compression = None
        for magic, ftype in magic_dict.items():
            if file_start.startswith(magic):
                compression = ftype
                break
        if compression == 'bz2':
            sequencefile = bz2.BZ2File(filename=bufferedfile)
            peek = sequencefile.peek(1)
        elif compression == 'gz':
            if not bufferedfile.seekable():
                bufferedfile.close()
                raise ValueError("gziped data not streamable, pipe "
                                 "through zcat first")
            peek = gzip.GzipFile(filename=filename).read(1)
            sequencefile = gzip.GzipFile(filename=filename)
            bufferedfile.close()
        else:
            peek = bufferedfile.peek(1)
            sequencefile = bufferedfile

        iter_fn = None
        try:
            first_char = peek[0]
        except IndexError as err:
            sequencefile.close()
            return []  # empty file

        try:
            first_char = chr(first_char)
        except TypeError:
            pass

        if first_char == '>':
            iter_fn = fasta_iter
        elif first_char == '@':
            iter_fn = fastq_iter

        if iter_fn is None:
            sequencefile.close()
            raise ValueError("unknown file format for '%s'" % filename)

        self.sequencefile = sequencefile
        return iter_fn(sequencefile, *args, **kwargs)

    def __enter__(self):
        return self.iter_fn

    def __exit__(self, *exc_info):
        self.close()

    def __iter__(self):
        if self.iter_fn:
            return self.iter_fn
        return iter(())

    def close(self):
        if self.sequencefile is not None:
            self.sequencefile.close()


class ScreedDB(MutableMapping):

    """
    Core on-disk dictionary interface for reading screed databases. Accepts a
    path string to a screed database
    """

    def __init__(self, filepath):
        self._db = None
        try:
            sqlite3
        except NameError:
            raise Exception("error: sqlite3 is needed for this " +
                            "functionality, but is not installed.")

        self._filepath = filepath
        if not self._filepath.endswith(DBConstants.fileExtension):
            self._filepath += DBConstants.fileExtension

        if not os.path.exists(self._filepath):
            raise ValueError('No such file: %s' % self._filepath)

        self._db = sqlite3.connect(self._filepath)
        cursor = self._db.cursor()

        # Make sure the database is a prepared screed database
        query = "SELECT name FROM sqlite_master WHERE type='table' "\
                "ORDER BY name"
        res = cursor.execute(query)
        try:
            dictionary_table, = res.fetchone()
            admin_table, = res.fetchone()

            if dictionary_table != DBConstants._DICT_TABLE:
                raise TypeError
            if admin_table != DBConstants._SCREEDADMIN:
                raise TypeError

        except TypeError:
            self._db.close()
            raise TypeError("Database %s is not a proper screed database"
                            % self._filepath)

        nothing = res.fetchone()
        if nothing is not None:
            self._db.close()
            raise TypeError("Database %s has too many tables." % filename)

        # Store the fields of the admin table in a tuple
        query = "SELECT %s, %s FROM %s" % \
            (DBConstants._FIELDNAME,
             DBConstants._ROLENAME,
             DBConstants._SCREEDADMIN)
        res = cursor.execute(query)
        self.fields = tuple([(str(field), role) for field, role in res])

        # Indexed text column for querying, search fields to find
        self._queryBy = self.fields[1][0]
        for fieldname, role in self.fields:
            if role == DBConstants._INDEXED_TEXT_KEY:
                self._queryBy = fieldname

        # Sqlite PRAGMA settings for speed
        cursor.execute("PRAGMA cache_size=2000")

        # Retrieve the length of the database
        query = 'SELECT MAX(%s) FROM %s' % (DBConstants._PRIMARY_KEY,
                                            DBConstants._DICT_TABLE)
        self._len, = cursor.execute(query).fetchone()

    def __del__(self):
        """
        Alias for close()
        """
        self.close()

    def close(self):
        """
        Closes the sqlite database handle
        """
        if self._db is not None:
            self._db.close()
            self._db = None

    def __getitem__(self, key):
        """
        Retrieves from database the record with the key 'key'
        """
        cursor = self._db.cursor()
        key = str(key)  # So lazy retrieval objectes are evaluated
        query = 'SELECT %s FROM %s WHERE %s=?' % (self._queryBy,
                                                  DBConstants._DICT_TABLE,
                                                  self._queryBy)
        res = cursor.execute(query, (key,))
        if res.fetchone() is None:
            raise KeyError("Key %s not found" % key)
        return screedRecord._buildRecord(self.fields, self._db,
                                         key,
                                         self._queryBy)

    def values(self):
        """
        Retrieves all records from the database and returns them as a list
        """
        return list(self.itervalues())

    def items(self):
        """
        Retrieves all records from the database and returns them as a list of
        (key, record) tuple pairs
        """
        return list(self.iteritems())

    def loadRecordByIndex(self, index):
        """
        Retrieves record from database at the given index
        """
        cursor = self._db.cursor()
        index = int(index) + 1  # Hack to make indexing start at 0
        query = 'SELECT %s FROM %s WHERE %s=?' % (DBConstants._PRIMARY_KEY,
                                                  DBConstants._DICT_TABLE,
                                                  DBConstants._PRIMARY_KEY)
        res = cursor.execute(query, (index,))
        if res.fetchone() is None:
            raise KeyError("Index %d not found" % index)
        return screedRecord._buildRecord(self.fields, self._db,
                                         index,
                                         DBConstants._PRIMARY_KEY)

    def __len__(self):
        """
        Returns the number of records in the database
        """
        return self._len

    def keys(self):
        """
        Returns a list of keys in the database
        """
        return list(self.iterkeys())

    def __repr__(self):
        """
        Returns a string with some general information about the database
        """
        return "<%s, '%s'>" % (self.__class__.__name__,
                               self._filepath)

    def itervalues(self):
        """
        Iterator over records in the database
        """
        for index in range(1, self.__len__() + 1):
            yield screedRecord._buildRecord(self.fields, self._db,
                                            index,
                                            DBConstants._PRIMARY_KEY)

    def iterkeys(self):
        """
        Iterator over keys in the database
        """
        cursor = self._db.cursor()
        query = 'SELECT %s FROM %s ORDER BY id' % (
            self._queryBy, DBConstants._DICT_TABLE)
        for key, in cursor.execute(query):
            yield key

    def __iter__(self):
        return self.iterkeys()

    def iteritems(self):
        """
        Iterator returning a (index, record) pairs
        """
        for v in self.itervalues():
            yield v[DBConstants._PRIMARY_KEY], v

    def has_key(self, key):
        """
        Returns true if given key exists in database, false otherwise
        """
        return key in self

    def copy(self):
        """
        Returns shallow copy
        """
        return self

    def __contains__(self, key):
        """
        Returns true if given key exists in database, false otherwise
        """
        cursor = self._db.cursor()
        query = 'SELECT %s FROM %s WHERE %s = ?' % \
                (self._queryBy, DBConstants._DICT_TABLE, self._queryBy)
        if cursor.execute(query, (key,)).fetchone() is None:
            return False
        return True

    # Here follow the methods that are not implemented

    def __setitem__(self, something):
        """
        Not implemented (Read-only database)
        """
        raise NotImplementedError

    def __delitem__(self, something):
        """
        Not implemented (Read-only database)
        """
        raise NotImplementedError

    def clear(self):
        """
        Not implemented (Read-only database)
        """
        raise NotImplementedError

    def update(self, something):
        """
        Not implemented (Read-only database)
        """
        raise NotImplementedError

    def setdefault(self, something):
        """
        Not implemented (Read-only database)
        """
        raise NotImplementedError

    def pop(self):
        """
        Not implemented (Read-only database)
        """
        raise NotImplementedError

    def popitem(self):
        """
        Not implemented (Read-only database)
        """
        raise NotImplementedError



import screed
from screed.DBConstants import fileExtension
import os
from . import screed_tst_utils as utils
import shutil


class nostring:
    def __str__(self):
        return ""

    def __repr__(self):
        return ""


class test_comparisons():

    def setup_method(self):
        self._testfile = utils.get_temp_filename('test.fa')
        shutil.copy(utils.get_test_data('test.fa'), self._testfile)
        screed.read_fasta_sequences(self._testfile)

        self._db = screed.ScreedDB(self._testfile)
        self._ns = nostring()

    def test_eq(self):
        for k in self._db:
            record = self._db.get(k)
            res = (record.sequence == self._ns)
            assert res is False, res

    def test_neq(self):
        for k in self._db:
            record = self._db.get(k)
            res = (record.sequence != self._ns)
            assert res is True, res

    def test_comp_greateq(self):
        for k in self._db:
            record = self._db.get(k)
            res = (record.sequence >= self._ns)
            assert res is True, res

    def test_comp_lesseq(self):
        for k in self._db:
            record = self._db.get(k)
            res = (record.sequence <= self._ns)
            assert res is False, res

    def test_comp_less(self):
        for k in self._db:
            record = self._db.get(k)
            res = (record.sequence < self._ns)
            assert res is False, res

    def test_comp_great(self):
        for k in self._db:
            record = self._db.get(k)
            res = (record.sequence > self._ns)
            assert res is True, res

    def teardown(self):
        os.unlink(self._testfile + fileExtension)



#!/usr/bin/env python
# Copyright (c) 2016, The Regents of the University of California.

import timeit
import sys
import os

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print "Usage: %s <filename>" % sys.argv[0]
        exit(1)

    screedFile = sys.argv[1]
    if not os.path.isfile(screedFile):
        print "No such file: %s" % screedFile
        exit(1)

    runStatement = """
for i in xrange(0, 100000):
    entry = str(db[random.choice(keys)].sequence)
"""

    setupStatement = """
import os, sys
import random
thisdir = sys.path[0]
libdir = os.path.abspath(os.path.join(thisdir, '..'))
sys.path.insert(0, libdir)
import screed
db = screed.openscreed.ScreedDB('%s')
keys = db.keys()
""" % screedFile

    t = timeit.Timer(runStatement, setupStatement)

    print "[SCREED RUN]%s:" % screedFile
    print t.repeat(2, 1)



from __future__ import absolute_import, unicode_literals, print_function
from screed import Record
import pytest


def test_create_quality_none():
    r = Record(name='foo', sequence='ATGACG', quality=None)
    assert not hasattr(r, 'quality')


def test_len():
    r = Record(name='foo', sequence='ATGACG')
    assert len(r) == 6


# copied over from khmer tests/test_read_parsers.py
def test_read_type_basic():
    name = "895:1:1:1246:14654 1:N:0:NNNNN"
    sequence = "ACGT"
    r = Record(name, sequence)

    assert r.name == name
    assert r.sequence == sequence
    assert not hasattr(r, 'quality'), x
    assert not hasattr(r, 'annotations'), x


# copied over from khmer tests/test_read_parsers.py
def test_read_type_attributes():
    r = Record(sequence='ACGT', quality='good', name='1234', annotations='ann')
    assert r.sequence == 'ACGT'
    assert r.quality == 'good'
    assert r.name == '1234'
    assert r.annotations == 'ann'



# Copyright (c) 2016, The Regents of the University of California.

import gc
import os
import sys
import tarfile
import urllib

thisdir = os.path.dirname(__file__)
libdir = os.path.abspath(os.path.join(thisdir, '..', 'screed'))
sys.path.insert(0, libdir)

from screed import read_fastq_sequences  # nopep8
from screed import read_fasta_sequences  # nopep8
from screed import ScreedDB  # nopep8

tests22 = os.path.join(thisdir,  's_2_2_sequence.fastq')
tests31 = os.path.join(thisdir, 's_3_1_sequence.fastq')
tests42 = os.path.join(thisdir, 's_4_2_sequence.fastq')
pongo = os.path.join(thisdir, 'Pongo_pygmaeus.PPYG2.50.cdna.abinitio.fa')
tri = os.path.join(thisdir, 'triCas2.fa')
mus = os.path.join(thisdir, 'Mus_musculus.NCBIM37.50.dna_rm.chromosome.9.fa')
xeno = os.path.join(thisdir, 'Xenopus_tropicalis.JGI4.1.50.dna.toplevel.fa')
sorex = os.path.join(thisdir, 'Sorex_araneus.COMMON_SHREW1.53.dna.toplevel.fa')


def getfile(f):
    """
    Downloads and extracts the given file
    """
    filetype = f[1]
    filename = "%s.tar.gz" % f[0]
    urlname = os.path.split(filename)[1]
    base_url = 'http://lyorn.idyll.org/~nolleyal/genomes/%s/%s' % \
        (filetype, urlname)

    fp = open(filename, 'wb')
    try:
        up = urllib.urlopen(base_url)
    except IOError:
        raise IOError("Error downloading testfiles, are you connected to "
                      "the internet?")
    fp.write(up.read())
    fp.close()

    tar = tarfile.open(filename)
    tar.extractall(path=thisdir)
    tar.close()
    os.unlink(filename)
    return


def setup():
    # Create databases
    endings = ['_screed']
    filenames = [
        (tests22, 'fastq'), (tests31, 'fastq'), (tests42, 'fastq'),
        (pongo, 'fasta'), (tri, 'fasta'), (mus, 'fasta'), (xeno, 'fasta'),
        (sorex, 'fasta')
    ]
    for f in filenames:
        fname = f[0]
        if not os.path.isfile(fname):  # Download files if necessary
            getfile(f)
        parser = None
        if f[1] == 'fasta':
            parser = read_fasta_sequences
        elif f[1] == 'fastq':
            parser = read_fastq_sequences
        created = True
        for end in endings:
            if not os.path.isfile(fname + end):
                created = False
        if not created:
            parser(fname)


class Test_s22_fastq:
    """
    Test screed methods on the s22 fastq file
    """
    def setup(self):
        self.db = ScreedDB(tests22 + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['HWI-EAS_4_PE-FC20GCB:2:1:492:573/2'] = {
            'id': 0,
            'annotations': '',
            'quality': 'AA7AAA3+AAAAAA.AAA.;7;AA;;;;*;<1;<<<',
            'name': 'HWI-EAS_4_PE-FC20GCB:2:1:492:573/2',
            'sequence': 'ACAGCAAAATTGTGATTGAGGATGAAGAACTGCTGT'}

        testcases['HWI-EAS_4_PE-FC20GCB:2:162:131:826/2'] = {
            'id': 1895228,
            'annotations': '',
            'quality': 'AAAAAAAAAAAAAAAAAAAAAA+@6=7A<05<*15:',
            'name': 'HWI-EAS_4_PE-FC20GCB:2:162:131:826/2',
            'sequence': 'ATGAATACAAACAATGCGGCAGTCATAATGCCCCTC'}

        testcases['HWI-EAS_4_PE-FC20GCB:2:330:88:628/2'] = {
            'id': 3790455,
            'annotations': '',
            'quality': 'AA;AA??A5A;;+AA?AAAA;AA;9AA.AA?????9',
            'name': 'HWI-EAS_4_PE-FC20GCB:2:330:88:628/2',
            'sequence': 'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACAAA'}

        testcases['HWI-EAS_4_PE-FC20GCB:2:4:707:391/2'] = {
            'id': 29999,
            'annotations': '',
            'quality': 'AAAAAAAAAA@<)A*AAA6A::<@AA<>A>-8?>4<',
            'name': 'HWI-EAS_4_PE-FC20GCB:2:4:707:391/2',
            'sequence': 'ATTAATCTCCAGTTTCTGGCAAACATTCAGGCCATT'}

        testcases['HWI-EAS_4_PE-FC20GCB:2:36:158:208/2'] = {
            'id': 342842,
            'annotations': '',
            'quality': 'AA5?AAAAA?AAAA5?AAA5A???5A>AAA4?;.;;',
            'name': 'HWI-EAS_4_PE-FC20GCB:2:36:158:208/2',
            'sequence': 'TTTCCCTACAGAAGTGTCTGTACCGGTAATAAAGAA'}

        for case in testcases:
            assert testcases[case] == self.db[case]


class Test_s31_fastq:
    """
    Test screed methods on the s31 fastq file
    """
    def setup(self):
        self.db = ScreedDB(tests31 + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['HWI-EAS_4_PE-FC20GCB:3:1:71:840/1'] = {
            'id': 0,
            'annotations': '',
            'quality': 'CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC',
            'name': 'HWI-EAS_4_PE-FC20GCB:3:1:71:840/1',
            'sequence': 'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'}

        testcases['HWI-EAS_4_PE-FC20GCB:3:330:957:433/1'] = {
            'id': 4439695,
            'annotations': '',
            'quality': 'AAAAAAAAAAA<A?<AA<AAAAA?AAA?<:*??&::',
            'name': 'HWI-EAS_4_PE-FC20GCB:3:330:957:433/1',
            'sequence': 'CTTTGTGGAGAAGAGGGCGTGGGCAAGGCACTGATA'}

        testcases['HWI-EAS_4_PE-FC20GCB:3:166:443:410/1'] = {
            'id': 2219847,
            'annotations': '',
            'quality': 'AAAAAAAAAAAAAAAAAAAAAAAA6<@AA959???%',
            'name': 'HWI-EAS_4_PE-FC20GCB:3:166:443:410/1',
            'sequence': 'TGGCATTCGCACACATCATGATGGTGCTGACCGTAA'}

        testcases['HWI-EAS_4_PE-FC20GCB:3:1:803:878/1'] = {
            'id': 2999,
            'annotations': '',
            'quality': '?6AAA6A<A6AA<<AA?A&A066/6:/&?&1191+0',
            'name': 'HWI-EAS_4_PE-FC20GCB:3:1:803:878/1',
            'sequence': 'AAGATGCTGTAGTGGCCGCATGTGTAATAGGCTTTA'}

        testcases['HWI-EAS_4_PE-FC20GCB:3:245:54:506/1'] = {
            'id': 3329772,
            'annotations': '',
            'quality': "AAAAAAAAAAAAAAAA>A+AAA+@AA+A>A%8*?'%",
            'name': 'HWI-EAS_4_PE-FC20GCB:3:245:54:506/1',
            'sequence': 'CTTCGTTGCTGTTTATCAGTAACTTTTTCTGGCTAG'}

        for case in testcases:
            assert testcases[case] == self.db[case]


class Test_s42_fastq:
    """
    Test screed methods on the s42 fastq file
    """
    def setup(self):
        self.db = ScreedDB(tests42 + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['HWI-EAS_4_PE-FC20GCB:4:1:257:604/2'] = {
            'id': 0,
            'annotations': '',
            'quality': 'AAAAAAAA:4>>AAA:44>>->-&4;8+8826;66.',
            'name': 'HWI-EAS_4_PE-FC20GCB:4:1:257:604/2',
            'sequence': 'TGTGGATAGTCGCCCGTGATGGCGTCGAAGTTCCGG'}

        testcases['HWI-EAS_4_PE-FC20GCB:4:330:96:902/2'] = {
            'id': 4148632,
            'annotations': '',
            'quality': 'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA??????',
            'name': 'HWI-EAS_4_PE-FC20GCB:4:330:96:902/2',
            'sequence': 'CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC'}

        testcases['HWI-EAS_4_PE-FC20GCB:4:166:158:532/2'] = {
            'id': 2074316,
            'annotations': '',
            'quality': 'AAAAAAA?A?AAAAAAA?A>A?A?AAAAAA?.<?-?',
            'name': 'HWI-EAS_4_PE-FC20GCB:4:166:158:532/2',
            'sequence': 'ATCGCCAATGCCCAGGCCTGGTTCTCTTTAACCTAT'}

        testcases['HWI-EAS_4_PE-FC20GCB:4:1:332:634/2'] = {
            'id': 3000,
            'annotations': '',
            'quality': '?A?AAAAAAAAA8>AAAAAA*AA?A?AA.?)<9)9?',
            'name': 'HWI-EAS_4_PE-FC20GCB:4:1:332:634/2',
            'sequence': 'ACCGTGCCAGATCAGAACCTAGTGGCGATTCCAATT'}

        testcases['HWI-EAS_4_PE-FC20GCB:4:242:843:13/2'] = {
            'id': 3111474,
            'annotations': '',
            'quality': "ABAAACA?CAAA??%A;2A;/5/&:?-*1-'11%71",
            'name': 'HWI-EAS_4_PE-FC20GCB:4:242:843:13/2',
            'sequence': 'GTTTCTATATTCTGGCGTTAGTCGTCGCCGATAATT'}

        for case in testcases:
            assert testcases[case] == self.db[case]


class Test_po_fasta:
    """
    Test screed methods on the pongo fasta file
    """
    def setup(self):
        self.db = ScreedDB(pongo + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['GENSCAN00000032971'] = {
            'id': 0,
            'description': 'cdna:Genscan chromosome:PPYG2:6_qbl_hap2_random'
            ':95622:98297:1',
            'name': 'GENSCAN00000032971',
            'sequence': 'ATGGCGCCCCGAACCCTCCTCCTGCTGCTCTCGGCGGCCCTGGCCCCGAC'
            'CGAGACCTGG'}
        testcases['GENSCAN00000042282'] = {
            'id': 53997,
            'description': 'cdna:Genscan chromosome:PPYG2:1:229892060:22989'
            '2800:1',
            'name': 'GENSCAN00000042282',
            'sequence': 'ATGATGCCATTGCAAGGACCCTCTGCAGGGCCTCAGTCCCGAGGATGGCA'
            'CACAGCCTTC'}
        testcases['GENSCAN00000051311'] = {
            'id': 30780,
            'description': 'cdna:Genscan chromosome:PPYG2:10:132962172:132'
            '962871:1',
            'name': 'GENSCAN00000051311',
            'sequence': 'ATGACCCAGCCACCTACCAGGCCGCTCTGCAGACCCCCCACGGGAGCAGC'
            'CTCTGCCCCC'}
        testcases['GENSCAN00000006030'] = {
            'id': 1469,
            'description': 'cdna:Genscan chromosome:PPYG2:14_random:1765749'
            ':1766075:-1',
            'name': 'GENSCAN00000006030',
            'sequence': 'ATGTGTGGCAACAAGGGCATTTCTGCCTTCCCTGAATCAGACCACCTTTT'
            'CACATGGGTA'}
        testcases['GENSCAN00000048263'] = {
            'id': 43029,
            'description': 'cdna:Genscan chromosome:PPYG2:6:100388173:10048'
            '5454:-1',
            'name': 'GENSCAN00000048263',
            'sequence': 'ATGTGTCCCTTTGAATATGCCGGAGAACAACAGTTGCCATGGATGTGTTC'
            'TGGGGAGCCC'}

        for case in testcases:
            assert testcases[case]['name'] == self.db[case]['name']
            assert testcases[case]['description'] == \
                self.db[case]['description']
            assert str(self.db[case]['sequence']).startswith(
                testcases[case]['sequence']
            )


class Test_mus_fasta:
    """
    Test screed methods on the mus_musculus fasta file
    """
    def setup(self):
        self.db = ScreedDB(mus + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['9'] = {
            'id': 0,
            'description': 'dna_rm:chromosome chromosome:NCBIM37:9:1:124076'
                           '172:1',
            'name': '9',
            'sequence': 'NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN'
                        'NNNNNNNNNN'
        }

        for case in testcases:
            assert testcases[case]['name'] == self.db[case]['name']
            assert testcases[case]['description'] == \
                self.db[case]['description']
            assert str(self.db[case]['sequence']).startswith(
                testcases[case]['sequence']
            )


class Test_tri_fasta:
    """
    Test screed methods on the tri fasta file
    """
    def setup(self):
        self.db = ScreedDB(tri + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['singleUn_100'] = {
            'id': 0,
            'description': '',
            'name': 'singleUn_100',
            'sequence': 'TTTAAACACGTGTCCGCGCCATTTTTTTATTTATTTACCGATCAAGTGCA'}
        testcases['singleUn_9'] = {
            'id': 2210,
            'description': '',
            'name': 'singleUn_9',
            'sequence': 'TTTAATTTTTTTACAACTCAAAATTTTGAGTAGTGTTTTAAATAGTACAC'}
        testcases['ChLG6'] = {
            'id': 2016,
            'description': '',
            'name': 'ChLG6',
            'sequence': 'CAAAAAAATTCATAACTCAAAAACTAAAAGTCGTAGAGCAATGCGGTTTG'}
        testcases['singleUn_286'] = {
            'id': 186,
            'description': '',
            'name': 'singleUn_286',
            'sequence': 'AAACTAAAACATCCTTTTCAGCATATTATTTGTTATATTTAAAAAAAAAC'}
        testcases['ChLG9'] = {
            'id': 2019,
            'description': '',
            'name': 'ChLG9',
            'sequence': 'CTGCCGATAATATTTCCTACCAGAAATAACCAATTTATTTTACGTATTAC'}

        for case in testcases:
            assert testcases[case]['name'] == self.db[case]['name']
            assert testcases[case]['description'] == \
                self.db[case]['description']
            assert str(self.db[case]['sequence']).startswith(
                testcases[case]['sequence']
            )


class Test_xeno_fasta:
    """
    Test screed methods on the xeno fasta file
    """
    def setup(self):
        self.db = ScreedDB(xeno + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['scaffold_20095'] = {
            'id': 0,
            'description': 'dna:scaffold scaffold:JGI4.1:scaffold_20095:1:2'
                           '001:1',
            'name': 'scaffold_20095',
            'sequence': 'GATGAGATCACCTTTCATGCTTTTTGTATCCCTATTATCTAGAGACAACAA'
                        'ATCAGTTGC'}
        testcases['scaffold_1'] = {
            'id': 19500,
            'description': 'dna:scaffold scaffold:JGI4.1:scaffold_1:1:781781'
                           '4:1',
            'name': 'scaffold_1',
            'sequence': 'CCTCCCTTTTTGGCTGTCTTTTCACTGTATCATAGCCTGGCGTGAACCAAG'
                        'CCTCAAAAA'}
        testcases['scaffold_271'] = {
            'id': 19230,
            'description': 'dna:scaffold scaffold:JGI4.1:scaffold_271:1:156'
                           '7461:1',
            'name': 'scaffold_271',
            'sequence': 'CGATTTTTGCGGAAAAACGCGAGTTTTTGGTAGCCATTCCGAAAGTTGCGA'
                        'TTTTTTGTA'}
        testcases['scaffold_19901'] = {
            'id': 329,
            'description': 'dna:scaffold scaffold:JGI4.1:scaffold_19901:1:22'
                           '56:1',
            'name': 'scaffold_19901',
            'sequence': 'ATACCGCAAAGGTTTCTTTCTTCTCAGTGCTCCATGCTGCCTCTCTTGTTT'
                        'TGCCTCCCT'}
        testcases['scaffold_95'] = {
            'id': 19408,
            'description': 'dna:scaffold scaffold:JGI4.1:scaffold_95:1:28996'
                           '70:1',
            'name': 'scaffold_95',
            'sequence': 'CCCTCCTGGTGATCCCACTTCAATCTCCCCATAGGCACACATCACTTCTAG'
                        'CAGTTCACA'}

        for case in testcases:
            assert testcases[case]['name'] == self.db[case]['name']
            assert testcases[case]['description'] == \
                self.db[case]['description']
            assert str(self.db[case]['sequence']).startswith(
                testcases[case]['sequence']
            )


class Test_sorex_fasta:
    """
    Test screed methods on the sorex fasta file
    """
    def setup(self):
        self.db = ScreedDB(sorex + '_screed')

    def tearDown(self):
        del self.db
        gc.collect()

    def test_iteration(self):
        """
        Runs through the database, accessing each element by index and then by
        name
        """
        for idx in xrange(0, len(self.db)):
            rcrd = self.db.loadRecordByIndex(idx)
            nameRcrd = self.db[rcrd.name]
            assert rcrd == nameRcrd

    def test_dict_stuff(self):
        """
        Tests some dictionary methods on the database
        """
        keys = self.db.keys()
        ikeys = list(self.db.iterkeys())
        assert sorted(keys) == sorted(ikeys)
        del keys
        del ikeys
        gc.collect()

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        assert self.db.get('FOO') is None
        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.update({})
            assert 0
        except AttributeError:
            pass

        try:
            db.clear()
            assert 0
        except AttributeError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except AttributeError:
            pass

        try:
            db.pop()
            assert 0
        except AttributeError:
            pass

        try:
            db.popitem()
            assert 0
        except AttributeError:
            pass

    def test_certain_records(self):
        """
        Pulls first, last, middle and few other records out of database and
        compares them to known quantities
        """
        testcases = {}
        testcases['scaffold_93039'] = {
            'id': 0,
            'description': 'dna:scaffold scaffold:COMMON_SHREW1:scaffold_93'
                           '039:1:203:1',
            'name': 'scaffold_93039',
            'sequence': 'GCTGAGCCTTGTAGTTCTGCTCCCTTTGACTGACGGCCCACTATGGACCG'
                        'GAAAAACTAC'}

        testcases['scaffold_107701'] = {
            'id': 1,
            'description': 'dna:scaffold scaffold:COMMON_SHREW1:scaffold_10'
                           '7701:1:203:1',
            'name': 'scaffold_107701',
            'sequence': 'TAAACCCAAAATAAACATTCCCCAAATTATATTTCTTCCTTTCCTTCTGA'
                        'ATAAAAGAAA'}

        testcases['GeneScaffold_6994'] = {
            'id': 243135,
            'description': 'dna:genescaffold genescaffold:COMMON_SHREW1:Gen'
                           'eScaffold_6994:1:2349312:1',
            'name': 'GeneScaffold_6994',
            'sequence': 'TATTGAGAGAAGTGGGAACTTCTCTAGTGGTGGGGTATGGTGATGGAATG'
                        'ATGTATGAAT'}

        testcases['scaffold_118324'] = {
            'id': 13823,
            'description': 'dna:scaffold scaffold:COMMON_SHREW1:scaffold_11'
                           '8324:1:884:1',
            'name': 'scaffold_118324',
            'sequence': 'CAGCCCCCTGCAACAAATTTTATACTCTAGAAACAGTTTAATGGCTGTTG'
                        'GAATATTTCC'}

        testcases['scaffold_92895'] = {
            'id': 14573,
            'description': 'dna:scaffold scaffold:COMMON_SHREW1:scaffold_92'
                           '895:1:890:1',
            'name': 'scaffold_92895',
            'sequence': 'GGGAAGCTTGCAAGGCTGTCCCATGTGGGCAGGAAGCTCTCAGTAGCTTG'
                        'CCAGTTTCTC'}

        testcases['scaffold_62271'] = {
            'id': 37101,
            'description': 'dna:scaffold scaffold:COMMON_SHREW1:scaffold_62'
                           '271:1:1064:1',
            'name': 'scaffold_62271',
            'sequence': 'AGAGTATCTCCCCCACATGGCAGAGCCTGGCAAGCTACCCATGGCGTATT'
                        'CAATATGCCA'}

        for case in testcases:
            assert testcases[case]['name'] == self.db[case]['name']
            assert testcases[case]['description'] == \
                self.db[case]['description']
            assert str(self.db[case]['sequence']).startswith(
                testcases[case]['sequence']
            )



import os
import psycopg2
import pdbConstants

def create_db(fields, rcrditer):
    """
    Populates the pgres database with records from the record iter
    """
    
    conn = psycopg2.connect('dbname=%s user=%s' % (pdbConstants._DBNAME,
                                                   pdbConstants._USER))
    cur = conn.cursor()

    # Create the admin table
    cur.execute('CREATE TABLE %s (ID serial PRIMARY KEY, '\
               'FIELDNAME TEXT)' % pdbConstants._SCREEDADMIN)

    for attribute in fields:
        cur.execute("INSERT INTO %s (FIELDNAME) VALUES ('%s')" % \
            (pdbConstants._SCREEDADMIN, attribute))

    # Setup the dictionary table creation field substring
    createsub = ','.join(['%s TEXT' % field for field in fields])

    # Create the dictionary table
    cur.execute('CREATE TABLE %s (%s serial PRIMARY KEY, %s)' %
                (pdbConstants._DICT_TABLE, pdbConstants._PRIMARY_KEY,
                 createsub))

    # Attribute to index
    queryby = fields[0]

    # Make the index on the 'queryby' attribute
    cur.execute('CREATE UNIQUE INDEX %sidx ON %s(%s)' %
                (queryby, pdbConstants._DICT_TABLE, queryby))

    # Setup the 'perc' pgres substring
    perc = ', '.join(['%s' for i in range(len(fields))])

    # Setup the sql substring for inserting data into db
    fieldsub = ','.join(fields)

    # Pull data from rcrditer and store in database
    for record in rcrditer:
        data = tuple([record[key] for key in fields])
        cur.execute('INSERT INTO %s (%s) VALUES (%s)' %\
                    (pdbConstants._DICT_TABLE, fieldsub, perc),
                    data)

    conn.commit()
    cur.close()
    conn.close()

def droptables():
    """
    Drops tables in db 
    """
    conn = psycopg2.connect('dbname=%s user=%s' % (pdbConstants._DBNAME,
                                                   pdbConstants._USER))
    cur = conn.cursor()

    try:
        cur.execute('DROP TABLE %s;' % pdbConstants._DICT_TABLE)
    except:
        pass
    try:
        cur.execute('DROP TABLE %s;' % pdbConstants._SCREEDADMIN)
    except:
        pass

    conn.commit()
    cur.close()
    conn.close()



# Copyright (c) 2008-2015, Michigan State University
"""
Test `screed.open`.
"""

from __future__ import absolute_import

import os.path
import sys
import subprocess

from . import screed_tst_utils as utils
import screed
import screed.openscreed


def test_empty_open():
    filename = utils.get_test_data('empty.fa')
    assert len(list(screed.open(filename))) == 0


def test_open_maps_dash():
    """Test mapping of '-'."""
    #  pylint: disable=protected-access
    filename = '-'
    mapped = screed.openscreed._normalize_filename(filename)

    assert '/dev/stdin' == mapped


def test_open_stdin():
    """Test feeding data through stdin.

    Uses a subprocess with the data file directlyused as stdin."""
    filename1 = utils.get_test_data('test.fa')
    command = ["python", "-c", "from __future__ import print_function;"
               "import screed; print(list(screed.open('-')))"]
    with open(filename1, 'rb') as data_file:
        output = subprocess.Popen(command,
                                  stdin=data_file, universal_newlines=True,
                                  stdout=subprocess.PIPE).communicate()[0]
        assert "'name': 'ENSMICT00000012722'" \
            or "'name': u'ENSMICT00000012722'" in output, output


def test_simple_open():
    filename = utils.get_test_data('test.fa')

    n = -1
    for n, record in enumerate(screed.open(filename, parse_description=True)):
        assert record.name == 'ENSMICT00000012722'
        break
    assert n == 0, n


def test_simple_open_fq():
    filename = utils.get_test_data('test.fastq')

    n = -1
    for n, record in enumerate(screed.open(filename)):
        assert record.name == 'HWI-EAS_4_PE-FC20GCB:2:1:492:573/2'
        break
    assert n == 0


def test_gz_open():
    filename1 = utils.get_test_data('test.fa')
    filename2 = utils.get_test_data('test.fa.gz')
    for n, (r1, r2) in enumerate(zip(screed.open(filename1),
                                     screed.open(filename2))):
        assert r1.name == r2.name

    assert n > 0


def test_bz2_open():
    filename1 = utils.get_test_data('test.fa')
    filename2 = utils.get_test_data('test.fa.bz2')
    for n, (r1, r2) in enumerate(zip(screed.open(filename1),
                                     screed.open(filename2))):
        assert r1.name == r2.name

    assert n > 0


def test_gz_open_fastq():
    filename1 = utils.get_test_data('test.fastq')
    filename2 = utils.get_test_data('test.fastq.gz')
    for n, (r1, r2) in enumerate(zip(screed.open(filename1),
                                     screed.open(filename2))):
        assert r1.name == r2.name

    assert n > 0


def test_unknown_fileformat():

    try:
        screed.open(__file__)
    except ValueError as err:
        assert "unknown file format" in str(err)



# Copyright (c) 2008, Michigan State University.

"""
Defines some constant strings identifications used in multiple
files throughout screed
"""

# Name of table holding information about rest of db
_SCREEDADMIN = 'SCREEDADMIN'

# Names of _SCREEDADMIN columns
_FIELDNAME = 'FIELDNAME'
_ROLENAME = 'ROLE'
_PRIMARY_KEY = 'id'

# Names of roles
_STANDARD_TEXT = 'STANDARDATTR'
_SLICEABLE_TEXT = 'SLICEABLEATTR'
_INDEXED_TEXT_KEY = 'TEXTKEYATTR'
_PRIMARY_KEY_ROLE = 'INTKEYATTR'

# Name of table holding sequence information
_DICT_TABLE = 'DICTIONARY_TABLE'

# The file extension given to all screed databases
fileExtension = '_screed'



# Copyright (c) 2016, The Regents of the University of California.

from __future__ import absolute_import
from . import DBConstants
from .screedRecord import Record
from .utils import to_str

FieldTypes = (('name', DBConstants._INDEXED_TEXT_KEY),
              ('description', DBConstants._STANDARD_TEXT),
              ('sequence', DBConstants._SLICEABLE_TEXT))


def fasta_iter(handle, parse_description=False, line=None):
    """
    Iterator over the given FASTA file handle, returning records. handle
    is a handle to a file opened for reading
    """
    if line is None:
        line = handle.readline()

    while line:
        data = {}

        line = to_str(line.strip())
        if not line.startswith('>'):
            msg = f"Bad FASTA format: no '>' at beginning of line: {line}"
            raise IOError(msg)

        if parse_description:  # Try to grab the name and optional description
            try:
                data['name'], data['description'] = line[1:].split(' ', 1)
            except ValueError:  # No optional description
                data['name'] = line[1:]
                data['description'] = ''
        else:
            data['name'] = line[1:]
            data['description'] = ''

        data['name'] = data['name'].strip()
        data['description'] = data['description'].strip()

        # Collect sequence lines into a list
        sequenceList = []
        line = to_str(handle.readline())
        while line and not line.startswith('>'):
            sequenceList.append(line.strip())
            line = to_str(handle.readline())

        data['sequence'] = ''.join(sequenceList)
        yield Record(**data)



from __future__ import absolute_import, unicode_literals
import screed
from screed.DBConstants import fileExtension
from screed.screedRecord import write_fastx
import os
from io import StringIO
from io import BytesIO
from . import screed_tst_utils as utils
import shutil
import pytest


class FakeRecord(object):
    """Empty extensible object"""
    pass


def test_new_record():
    # test for a bug where the record dict was not reset after each
    # sequence load, leading to all records being identical if you
    # kept a handle on the returned dictionary.

    s = StringIO("@1\nACTG\n+\nAAAA\n@2\nACGG\n+\nAAAA\n")

    records = list(iter(screed.fastq.fastq_iter(s)))
    assert records[0]['name'] == '1'
    assert records[1]['name'] == '2'


def test_parse_description_true():
    # test for a bug where the record dict was not reset after each
    # sequence load, leading to all records being identical if you
    # kept a handle on the returned dictionary.

    s = StringIO("@1 FOO\nACTG\n+\nAAAA\n@2\nACGG\n+\nAAAA\n")

    records = list(iter(screed.fastq.fastq_iter(s, parse_description=True)))
    assert records[0]['name'] == '1'
    assert records[1]['name'] == '2'


def test_parse_description_false():
    # test for a bug where the record dict was not reset after each
    # sequence load, leading to all records being identical if you
    # kept a handle on the returned dictionary.

    s = StringIO("@1 FOO\nACTG\n+\nAAAA\n@2\nACGG\n+\nAAAA\n")

    records = list(iter(screed.fastq.fastq_iter(s, parse_description=False)))
    assert records[0]['name'] == '1 FOO'
    assert records[1]['name'] == '2'

    # also is default behavior
    s = StringIO("@1 FOO\nACTG\n+\nAAAA\n@2\nACGG\n+\nAAAA\n")

    records = list(iter(screed.fastq.fastq_iter(s)))
    assert records[0]['name'] == '1 FOO'
    assert records[1]['name'] == '2'


class Test_fastq(object):

    def setup_method(self):
        self._testfq = utils.get_temp_filename('test.fastq')
        shutil.copy(utils.get_test_data('test.fastq'), self._testfq)

        screed.read_fastq_sequences(self._testfq)
        self.db = screed.ScreedDB(self._testfq)

    def teardown_method(self):
        os.unlink(self._testfq + fileExtension)

    def test_length(self):
        assert len(self.db) == 125

    def test_keys(self):
        for key in self.db:
            assert key == self.db[key].name

    def test_id_retrieval(self):
        for key in self.db:
            record = self.db[key]
            intRcrd = self.db.loadRecordByIndex(record.id)
            assert record == intRcrd

    def test_contains_front(self):
        first = self.db[self.db.keys()[0]]
        assert first.id == 0
        assert first.name == 'HWI-EAS_4_PE-FC20GCB:2:1:492:573/2'
        assert first.sequence == 'ACAGCAAAATTGTGATTGAGGATGAAGAACTGCTGT'
        assert first.quality == 'AA7AAA3+AAAAAA.AAA.;7;AA;;;;*;<1;<<<'

    def test_contains_middle(self):
        middle = self.db[self.db.keys()[62]]
        assert middle.id == 62
        assert middle.name == 'HWI-EAS_4_PE-FC20GCB:2:1:245:483/2'
        assert middle.sequence == 'TGTCGAGCAAAGCAAAACAGGCGTAAAAATTGCCAT'
        assert middle.quality == 'AAAAAAAAAAAAAAAAAAAAA>AAAAAAAA?9>6><'

    def test_contains_end(self):
        end = self.db[self.db.keys()[124]]
        assert end.id == 124
        assert end.name == 'HWI-EAS_4_PE-FC20GCB:2:1:350:588/2'
        assert end.sequence == 'GGTACAAAATAGATGCTGGACTCTCCGAATCCTATA'
        assert end.quality == ';?5AAAAAAAAAA?A??;?AA;AAA>AAAA?4?844'

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert self.db.get('FOO') is None

        assert 'FOO' not in self.db

    def test_iterv(self):
        entries = []
        for entry in self.db:
            entries.append(self.db[entry])

        ivalues = list(self.db.itervalues())
        assert all(entry in ivalues for entry in entries)

    def test_iteri(self):
        for id, entry in self.db.iteritems():
            assert id == self.db[entry.name].id
            assert entry == self.db[entry.name]


def test_output_sans_desc():
    read = FakeRecord()
    read.name = 'foo'
    read.sequence = 'ATCG'
    read.quality = '####'

    fileobj = BytesIO()
    write_fastx(read, fileobj)
    assert fileobj.getvalue().decode('utf-8') == '@foo\nATCG\n+\n####\n'


def test_output_with_desc():
    read = FakeRecord()
    read.name = 'foo'
    read.description = 'bar'
    read.sequence = 'ATCG'
    read.quality = '####'

    fileobj = BytesIO()
    write_fastx(read, fileobj)
    assert fileobj.getvalue().decode('utf-8') == '@foo bar\nATCG\n+\n####\n'


def test_output_two_reads():
    fileobj = BytesIO()
    for i in range(2):
        read = FakeRecord()
        read.name = 'seq{}'.format(i)
        read.sequence = 'GATTACA' * (i + 1)
        read.quality = '#######' * (i + 1)
        write_fastx(read, fileobj)
    testoutput = ('@seq0\nGATTACA\n+\n#######\n'
                  '@seq1\nGATTACAGATTACA\n+\n##############\n')
    assert fileobj.getvalue().decode('utf-8') == testoutput


def test_output_bad_mode():
    read = FakeRecord()
    read.name = 'foo'
    read.description = 'bar'
    read.sequence = 'ATCG'
    read.quality = '####'

    fileobj = StringIO()
    with pytest.raises(AttributeError) as ae:
        write_fastx(read, fileobj)
    assert 'cannot call "write_fastx" on object' in str(ae)


def test_fastq_slicing():
    testfq = utils.get_temp_filename('test.fastq')
    shutil.copy(utils.get_test_data('test.fastq'), testfq)

    with screed.open(testfq) as sequences:
        record = next(sequences)

    trimmed = record[:10]
    assert trimmed['sequence'] == "ACAGCAAAAT"
    assert trimmed['quality'] == "AA7AAA3+AA"

    for s in (slice(5, 10), slice(2, 26), slice(5, -1, 2),
              slice(-2, -10, 1), slice(-1, 5, 2), slice(5)):
        trimmed = record[s]

        assert trimmed['name'] == record['name']
        assert trimmed.name == record.name

        assert trimmed['sequence'] == record['sequence'][s]
        assert trimmed.sequence == record.sequence[s]

        assert trimmed['quality'] == record['quality'][s]
        assert trimmed.quality == record.quality[s]



from __future__ import absolute_import
import os
import screed
from screed.DBConstants import fileExtension
from . import screed_tst_utils as utils
import shutil


class Test_dict_methods(object):

    """
    Make sure that screed returns sensible results for standard dictionary
    queries.
    """

    def setup_method(self):
        self._testfa = utils.get_temp_filename('test.fa')
        shutil.copy(utils.get_test_data('test.fa'), self._testfa)

        screed.read_fasta_sequences(self._testfa)
        self.db = screed.ScreedDB(self._testfa)

    def teardown_method(self):
        os.unlink(self._testfa + fileExtension)

    def test_iter_stuff(self):
        db = self.db
        keys = db.keys()
        ikeys = list(db.iterkeys())
        assert all(key in ikeys for key in keys)

        values = db.values()
        ivalues = list(db.itervalues())
        assert all(value in ivalues for value in values)

        items = db.items()
        iitems = list(db.iteritems())
        assert all(item in iitems for item in items)

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert db.get('FOO') is None

        assert 'FOO' not in self.db

    def test_contains(self):
        for k in self.db:
            assert k in self.db

        assert 'FOO' not in self.db

    def test_get(self):
        for k in self.db:
            record = self.db.get(k)
            assert record.name == k

            record = self.db[k]
            assert record.name == k

        try:
            self.db['FOO']
            assert False, "the previous line should raise a KeyError"
        except KeyError:
            pass

    def test_missing(self):
        """
        Make sure that unsupported dict attributes are actually missing.
        """
        db = self.db

        try:
            db.clear()
            assert 0
        except NotImplementedError:
            pass

        try:
            db.update({})
            assert 0
        except NotImplementedError:
            pass

        try:
            db.setdefault(None)
            assert 0
        except NotImplementedError:
            pass

        try:
            db.pop()
            assert 0
        except NotImplementedError:
            pass

        try:
            db.popitem()
            assert 0
        except NotImplementedError:
            pass



# Copyright (c) 2008-2010, Michigan State University.

from __future__ import absolute_import
from .openscreed import ScreedDB

_MAXLINELEN = 80
_null_quality = '\"'  # ASCII 34, e.g 75% chance of incorrect read


def GetComments(value):
    """
    Returns description or annotations attributes from given
    dictionary object
    """
    if 'description' in value:
        return value['description']
    elif 'annotations' in value:
        return value['annotations']
    else:
        return ''


def linewrap(longString):
    """
    Given a long string of characters, inserts newline characters
    every _MAXLINELEN characters
    """
    res = []
    begin = 0
    while begin < len(longString):
        res.append(longString[begin:begin + _MAXLINELEN])
        begin += _MAXLINELEN

    return '\n'.join(res)


def GenerateQuality(value):
    """
    Returns quality from value if it exists. Otherwise, makes
    a null quality. Quality is line wrapped to _MAXLINELEN
    either way
    """
    if 'quality' in value:
        return linewrap(value['quality'])

    return linewrap(_null_quality * len(str(value['sequence'])))


def ToFastq(dbFile, outputFile):
    """
    Opens the screed database file and attempts to dump it
    to a FASTQ-formatted text file
    """
    outFile = open(outputFile, 'wb')
    db = ScreedDB(dbFile)

    for n, value in enumerate(db.itervalues()):
        line = '@%s %s\n%s\n+\n%s\n' % (value['name'],
                                        GetComments(value),
                                        linewrap(str(value['sequence'])),
                                        GenerateQuality(value))
        outFile.write(line.encode('UTF-8'))
    db.close()
    outFile.close()

    return n + 1


def ToFasta(dbFile, outputFile):
    """
    Opens the screed database file and attempts to dump it
    to a FASTA-formatted text file
    """
    outFile = open(outputFile, 'wb')
    db = ScreedDB(dbFile)

    for n, value in enumerate(db.itervalues()):
        line = '>%s %s\n%s\n' % (value['name'], GetComments(value),
                                 linewrap(str(value['sequence'])))
        outFile.write(line.encode('UTF-8'))

    db.close()
    outFile.close()

    return n + 1



#!/usr/bin/env python
# Copyright (c) 2016, The Regents of the University of California.

import sys, os
import random

seqLength = (8000, 12000)

class collectionOFiles(object):
    def __init__(self, baseName, divisions, totalSize):
        self.baseName = baseName
        self.divisions = divisions
        self.totalSize = totalSize

        self.fileHandles = {}
        for i in range(0, divisions):
            filename = self.baseName + "_%d" % i
            fh = open(filename, "wb")
            divisor = 2 ** i

            self.fileHandles[filename]= (fh, self.totalSize/divisor, 0)

    def writeRecord(self, name, description, sequence):
        toRemove = []
        for filename in self.fileHandles:
            file, limit, count = self.fileHandles[filename]
            file.write("%s %s\n%s\n" % (name, description, sequence))
            count += 1
            if count >= limit:
                file.close()
                toRemove.append(filename)
            else:
                self.fileHandles[filename] = (file, limit, count)

        for fh in toRemove:
            self.fileHandles.pop(fh)

    def finished(self):
        return len(self.fileHandles) == 0

def genSeq(min, max):
    """
    Generates a sequence with min <= length <= max
    """
    choices = ['A','T','C','G']
    result = []
    length = random.randrange(min, max)
    for i in range(0, length):
        result.append(random.choice(choices))
        if i % 80 == 0:
            result.append('\n')
    return "".join(result)

def createFastaFiles(filename, size, divisions):
    cof = collectionOFiles(filename, divisions, size)
    counter = 0
    description="cdna:Genscan chromosome:PPYG2:6_qbl_hap2_random:95622:98297:1"
    while(not cof.finished()):
        name = ">GENSCAN00%d" % counter
        sequence = genSeq(seqLength[0], seqLength[1])
        cof.writeRecord(name, description, sequence)
        counter += 1
    return

if __name__ == '__main__':
    if len(sys.argv) != 4:
        print "Usage: <filename> <size> <divisions>"
        exit(1)

    filename = sys.argv[1]
    size = int(sys.argv[2])
    divisions = int(sys.argv[3])

    createFastaFiles(filename, size, divisions)



# Copyright (c) 2008-2015, Michigan State University
"""
Test the use of `screed.open` as a ContextManager.
"""

from . import screed_tst_utils as utils
import screed
import screed.openscreed


def test_empty_open():
    filename = utils.get_test_data('empty.fa')
    with screed.open(filename) as f:
        assert len(list(f)) == 0


def test_simple_open():
    filename = utils.get_test_data('test.fa')

    n = -1
    with screed.open(filename, parse_description=True) as f:
        for n, record in enumerate(f):
            assert record.name == 'ENSMICT00000012722'
            break

        assert n == 0, n


def test_simple_close():
    filename = utils.get_test_data('test.fa')

    n = -1
    f = screed.open(filename, parse_description=True)
    for n, record in enumerate(f):
        assert record.name == 'ENSMICT00000012722'
        break

    assert n == 0, n
    f.close()


def test_simple_open_fq():
    filename = utils.get_test_data('test.fastq')

    n = -1
    with screed.open(filename) as f:
        for n, record in enumerate(f):
            assert record.name == 'HWI-EAS_4_PE-FC20GCB:2:1:492:573/2'
            break

        assert n == 0


def test_gz_open():
    filename1 = utils.get_test_data('test.fa')
    filename2 = utils.get_test_data('test.fa.gz')
    with screed.open(filename1) as f1, screed.open(filename2) as f2:
        for n, (r1, r2) in enumerate(zip(f1, f2)):
            assert r1.name == r2.name

        assert n > 0


def test_bz2_open():
    filename1 = utils.get_test_data('test.fa')
    filename2 = utils.get_test_data('test.fa.bz2')
    with screed.open(filename1) as f1, screed.open(filename2) as f2:
        for n, (r1, r2) in enumerate(zip(f1, f2)):
            assert r1.name == r2.name

        assert n > 0


def test_gz_open_fastq():
    filename1 = utils.get_test_data('test.fastq')
    filename2 = utils.get_test_data('test.fastq.gz')
    with screed.open(filename1) as f1, screed.open(filename2) as f2:
        for n, (r1, r2) in enumerate(zip(f1, f2)):
            assert r1.name == r2.name

        assert n > 0


def test_unknown_fileformat():
    try:
        with screed.open(__file__):
            pass
    except ValueError as err:
        assert "unknown file format" in str(err)
